#    something which can be used to check if loaded
set common.vxi.loaded 1

#    some procedures used by all VXI card tasks

if {[info exists table] != 0} then {

#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"

#    create the lists "regnames", "widgetnames" and "parameternames" from the elements of "table"

set regnames {}
set widgetnames {}
set parameternames {}

set names [lsort [array names table]]
foreach w $names {
   set line $table($w)
   if {[lindex $line 12] != "nil"} then {
      lappend widgetnames $w
      lappend regnames [lindex $line 3]
      lappend parameternames [lindex $line 12]
      set param [lindex $line 0]
      set $param [lindex $line 11]
   }
}
foreach w $names {
   set line $table($w)
   if {[lindex $line 12] == "nil"} then {
      lappend widgetnames $w
      lappend regnames [lindex $line 3]
      set param [lindex $line 0]
      set $param [lindex $line 11]
   }
}

}


#    define some variables and give default values if they do not
#    already exist

if {[info exists seglist] == 0}  then {set seglist {???}; set segindex -1}
if {[info exists chanlist] == 0} then {set chanlist {???}; set chanindex -1}
if {[info exists chantype] == 0} then {set chantype ""}

if {[info exists registersavepatterns] == 0} then { set registersavepatterns {} }

if {[info exists allchannels] == 0}  then {set allchannels "\[?,??,???\]"}
if {[info exists allsegments] == 0}  then {set allsegments ?}
if {[info exists allchans] == 0} then {set allchans 0}
if {[info exists allsegs]  == 0} then {set allsegs 0}

if {[info exists chansel] == 0} then {set chansel 0}
if {[info exists segsel]  == 0} then {set segsel 0}

if {[info exists chanselection] == 0} then {set chanselection ""}
if {[info exists segselection]  == 0} then {set segselection ""}

if {[info exists electronicsvisible] == 0} then {set electronicsvisible 0}
if {[info exists suppressionvisible] == 0} then {set suppressionvisible 0}
if {[info exists lcavisible]         == 0} then {set lcavisible 0}

if {[info exists insp_table] == 0} then {set inspection 0} else {set inspection 1}
if {[info exists unshown] == 0} then {set unshown 1}
if {[info exists frameformat] == 0} then {set frameformat 1}

if {[info exists parameter] == 0} then {set parameter ENABLE}
if {[info exists register] == 0} then {set register Enable}


#
#    procedures used to convert a widget value into a parameter  ##############################
#


proc int-widget->param {p v s} {global $p; set $p $v}

proc hex-widget->param {p v s} {global $p; set $p $v}

proc str-widget->param {p v s} {global $p; set $p $v}

proc logic-widget->param {p v s} {global $p; set $p [expr $v & 1]}

proc widget->param {p v s} {global $p; set $p $v}

proc slider-widget->param {p v s} {
	global $p 
#	set oldv [set $p]

#	if {$v==$oldv+1} then {set v [expr $oldv+$s]}
#	if {$v==$oldv-1} then {set v [expr $oldv-$s]}
	set $p $v
   }

proc spare-widget->param {p v s} {global $p; puts stdout "spare-widget->param $p $v $s"}

proc null-repainter {p v u} {}


#
#    procedures used to convert a parameter into a widget value   ############################
#



proc param->int-widget {p v u} {return "$p :value $v" }

proc real-param->int-widget {p v u} {return "$p :value [truncate [expr $v+0.5]]"}

#proc param->logic-widget {p v u} {return "$p :value [expr $v & 1]" }
proc param->logic-widget {p v u} {
   if {$v == 0} then {set vl 0} else {set vl 1}
   return "$p :value $vl" 
}

proc param->str-widget {p v u} {return "$p :value $v" }

proc param->hex-str-widget {p v u} {format "%s :value 0x%x" $p $v }

proc param->hex-widget {p v u} {format "%s :value 0x%x" $p $v }


proc param->choice-widget {p v u} {}

proc param->analogue-insp-reg {p v u} {global analogueinsptable; lindex $analogueinsptable $v}

proc param->digital-insp-reg {p v u} {global digitalinsptable; lindex $digitalinsptable $v}

proc param->voltage-insp-reg {p v u} {global voltageinsptable; lindex $voltageinsptable $v}

proc param->suppression-reg {p v u} {global suppressiontable; lindex $suppressiontable $v}


proc param->reg {p v u} {return "$v"}

proc param->int-reg {p v u} {return "$v"}

proc param->unit-reg {p v u} {return "$v$u"}
proc param->unit-reg {p v u} {global $p; return "[set $p]$u"}

proc param->not-reg {p v u} {return "[expr $v^1]"}

proc param->spare-reg {p v u} {puts stderr "param->spare-reg $p $v $u"; return "$v"}


#
#    procedures used to covert a register value into a parameter    #############################
#


proc reg->param {p v} {global $p; set $p $v}

proc int-reg->param {p v} {global $p; set $p [scan-int $v]}

proc str-reg->param {p v} {global $p; set $p $v}

proc int-reg->choice-param {p v} {}

proc unit-reg->param {p v} {global $p; set $p [scan-int $v]}

proc not-reg->param {p v} {global $p; set $p [expr [scan-int $v]^1]}

proc spare-reg->param {p v} {
   global cardtype
   global $p
   if {$v == "0xffff"} then {set $p 0; return}
   if {$v == "0xffffffff"} then {set $p 0; return}
   switch $cardtype {
      Clover       {set $p [expr ((($v>>10)&0xf)-1)%5]}
      ClusterGe    {set $p [expr (((($v>>8)&0xff)-1)/3)+1]}
      ClusterBGO   {set $p [expr (((($v>>8)&0xff)-0x19)/2)+1]}
   }
}

proc digital-insp-reg->param {p v} {
   global digitalinsptable
   global $p
   set i [lsearch $digitalinsptable $v]
   if {$i<0} then {puts stderr "digital inspection param '$v' not recognised"; return}
   set $p $i   
}

proc analogue-insp-reg->param {p v} {
   global analogueinsptable
   global $p
   set i [lsearch $analogueinsptable $v]
   if {$i<0} then {puts stderr "analogue inspection param '$v' not recognised"; return}
   set $p $i   
}

proc voltage-insp-reg->param {p v} {
   global voltageinsptable
   global $p
   set i [lsearch $voltageinsptable $v]
   if {$i<0} then {puts stderr "voltage inspection param '$v' not recognised"; eturn}
   set $p $i   
}

proc suppression-reg->param {p v} {
   global debug suppressiontable
   global $p
   set i [lsearch $suppressiontable $v]
   if {$i<0} then {set i [lsearch $suppressiontable "File..."]}
   set $p $i   
}


#
#    procedures which convert between a channel and a widget    #########################
#
 

proc widget->channel {p v s} {
   global $p chanlist chanindex
   set chanindex $v
   make-myserver
   set $p [lindex $chanlist $v]
}

proc channel->widget {p v u} {return "($p :value [expr $v-1])"}


#
#    procedures used to create the "by parameter" windows       ###############################
#

#    card specific sizes

proc make-panel w {
   global debug frame table register unshown
   global cardtype chantype panel_w panel_h

   if {$debug} then {puts stderr "proc make-panel called with argument \{$w\}"}
     close-frame
     :modify PANEL :delete T
     :panel PANEL  :layout T :padx 5 :pady 5
       :layout H; :col-gap 0; :row-gap 0
   set line $table($w)
   set type [lindex $line 7]
   set min [lindex $line 8]
   set max [lindex $line 9]
   switch $type {
     :slider {
        switch $cardtype$chantype {
           CloverBGO -
           BgoBall -
           Ge    -
           BGO   {make-slider-panel-c $w $min $max}
           default {make-slider-panel-s $w $min $max}
        }
     }
     :slider-s {make-slider-panel-s $w $min $max}
     :slider-c {make-slider-panel-c $w $min $max}

     :checkbox {
        switch $cardtype$chantype {
           CloverBGO -
           BgoBall -
           Ge    -
           BGO   {make-checkbox-panel-c $w}
           default {make-checkbox-panel-s $w}
        }
     }
     :checkbox-s {make-checkbox-panel-s $w}
     :checkbox-c {make-checkbox-panel-c $w}

     :text {
        switch $cardtype$chantype {
           CloverBGO -
           BgoBall -
           Ge    -
           BGO   {make-text-panel-c $w $min}
           default {make-text-panel-s $w $min}
        }
     }
     :text-s {make-text-panel-s $w $min}
     :text-c {make-text-panel-c $w $min}

     :choice {
        switch $cardtype$chantype {
           CloverBGO -
           BgoBall -
           Ge    -
           BGO   {make-choice-panel-c $w $min $max}
           default {make-choice-panel-s $w $min $max}
        }
     }
     :choice-s {make-choice-panel-s $w $min $max}
     :choice-c {make-choice-panel-c $w $min $max}

   }
    repaint-parameter
    open-frame
    set unshown 1
}

proc make-slider-panel-c {w min max} {
   global debug env
   if {$debug} then {puts stderr "proc make-slider-panel called with arguments $w $min $max"}
	global chanlist seglist
        set i 0
	foreach c $chanlist {
            :slider [set w]_[set c]_ :valuex 60 :label "$c:" :min $min :max $max
            incr i
            if {$i == 4} then {:next-row T; set i 0}
       }
}


proc make-slider-panel-s {w min max} {
   global debug env
   if {$debug} then {puts stderr "proc make-slider-panel called with arguments $w $min $max"}
	global chanlist seglist
	foreach c $chanlist {
           foreach s $seglist {
              :slider [set w]_[set c]_[set s] :valuex 40 :label "$c$s:" :min $min :max $max :scale-length 50
           }
           :next-row T
       }
}

proc make-checkbox-panel-c {w} {
#    show only channel
	global debug chanlist seglist
        if {$debug} then {puts stderr "proc make-checkbox-panel called with arguments $w"}
        set i 0
	foreach c $chanlist {
            :checkbox [set w]_[set c]_ :valuex 40 :label "$c:" 
            incr i
            if {$i == 10} then {:next-row T; set i 0}
	}
}

proc make-checkbox-panel-s {w} {
#    show only segments
	global debug chanlist seglist
   if {$debug} then {puts stderr "proc make-checkbox-panel called with arguments $w"}
	foreach c $chanlist {
            foreach s $seglist {
	        :checkbox [set w]_[set c]_[set s] :valuex 80 :label "$c$s:" 
	    }
            :next-row T
	}
}

proc make-text-panel-c {w width} {
	global debug chanlist seglist
   if {$debug} then {puts stderr "proc make-text-panel called with arguments $w $width"}
        set i 0
	foreach c $chanlist {
            :text [set w]_[set c]_ :valuex 40 :label "$c:" :w $width 
            incr i
            if {$i == 10} then {:next-row T; set i 0}
	}
}

proc make-text-panel-s {w width} {
	global debug chanlist seglist
   if {$debug} then {puts stderr "proc make-text-panel called with arguments $w $width"}
	foreach c $chanlist {
		foreach s $seglist {
                       :text [set w]_[set c]_[set s] :valuex 80 :label "$c$s:" :w $width 
		}
                :next-row T
	}
}

proc make-choice-panel-c {w off on} {
   global debug chanlist seglist
   if {$debug} then {puts stderr "proc make-choice-panel called with arguments $w $off $on"}
        set i 0
	foreach c $chanlist {
		:choice [set w]_[set c]_ :valuex 40 :label "$c:" \
                               :strings [list $off $on]
            incr i
            if {$i == 8} then {:next-row T; set i 0}
	}
}

proc make-choice-panel-s {w off on} {
   global debug chanlist seglist
   if {$debug} then {puts stderr "proc make-choice-panel called with arguments $w $off $on"}
	foreach c $chanlist {
		foreach s $seglist {
	             	:choice [set w]_[set c]_[set s] :valuex 40 :label "$c$s:"  \
	          	               :strings [list $off $on]
		}
                :next-row T
	}
}


#
#    basic VXI register server access                 #################################
#


if {[info exists env(VXIServer)] == 0} then {set VXIServer VXI}  \
   else {set VXIServer $env(VXIServer)}
set resource $VXIServer
set myserver $VXIServer
set CurrentVXIServer $VXIServer

#    server for the Euroball inspection line router 
#    Note - default state here is that it does not exist

set MUXserver ""
set MUXrouting 0
proc connect-inspection args {}

if {[info exists VXIGlobal] == 0} then {set VXIGlobal "Global"}


proc make-reg-name {chan seg reg} {
  global VXIServer resource cardtype chantype segindex
  global VXIGlobal

  if {$resource != $VXIServer} then {set resource [VXImodule2crate $cardtype$chan]}
  if {$segindex == -1 || $seg == ""} then {
     if {$VXIGlobal == ""} then {
        return /$resource/$cardtype$chan.$reg
     } else {
        return /$resource/$cardtype$chan\[,.$VXIGlobal\].$reg
     }
  } else {
     if {$VXIGlobal == ""} then {
        return /$resource/$cardtype$chan\[,.$chantype$seg\].$reg
     } else {
        return /$resource/$cardtype$chan\[.$VXIGlobal,.$chantype$seg\].$reg
     }
  }
}

set userselection 0

proc write-reg {c s reg v g} {
    global userselection
    global chanindex segindex allchans allsegs allchannels allsegments
    global chanselection segselection
    global VXIServer resource cardtype chantype VXIGlobal

    set chan $c; set seg $s
    if {$g != 0} then {
       if {$chanindex >= 0} then {
          if {$allchans > 0} then {set chan $allchannels}
          if {$allchans < 0} then {set chan $chanselection}
       }
       if {$segindex >= 0} then {
          if {$allsegs > 0} then {set seg $allsegments}
          if {$allsegs < 0} then {set seg $segselection}
       }
    }

#    if there is a user selection of channel and segment to apply
#    then must do in 2 chunks because of limitations in the 
#    register server pattern match

   if {$userselection == 0} then {
      set rr [make-reg-name $chan $seg $reg]
      if {[llength [split "$rr" ?\[*]] == 1} then {
         EG "register :name '$rr' :value '$v'"
      } else {
         EG "register :pattern '$rr' :value '$v'"
      }
   } else {
      if {$resource != $VXIServer} then {set resource [VXImodule2crate $cardtype$chan]}
      if {"$VXIGlobal" != ""} then {
         set r1 "/$resource/$cardtype$chan.$VXIGlobal.$reg"
      } else {
         set r1 "/$resource/$cardtype$chan.$reg"
      }
      EG "register :pattern '$r1' :value '$v'"
      if {"$chantype$seg" != ""} then {
         set r2 "/$resource/$cardtype$chan.$chantype$seg.$reg"
      } else {
         set r2 "/$resource/$cardtype$chan.$reg"
      }
      if {"$r1" != "$r2"} then {EG "register :pattern '$r2' :value '$v'"}
   }
}

proc write-insp-reg {chan s reg v args} {
    global VXIServer resource cardtype VXIGlobal

     if {$resource != $VXIServer} then {set resource [VXImodule2crate $cardtype$chan]}

     if {$VXIGlobal == ""} then {
        set rr /$resource/$cardtype$chan\[,.$s\].$reg
     } else {
        set rr /$resource/$cardtype$chan\[,.$VXIGlobal\].$reg
     }

     EG "register :pattern '$rr' :value '$v'"
}

proc write-attribute {c s reg v g} {
    global chanindex segindex allchans allsegs allchannels allsegments

    set chan $c; set seg $s
    if {$g != 0} then {
       if {$chanindex >= 0} then {
          if {$allchans != 0} then {set chan $allchannels}
       }
       if {$segindex >= 0} then {
          if {$allsegs  != 0} then {set seg $allsegments}
       }
    }
   set rr [make-reg-name $chan $seg $reg]
   EG "register :pattern '$rr' :object '$v'"
}

proc write-vxi-reg {reg v} {
   global chanlist chanindex seglist segindex
    
   if {$chanindex >= 0} then {set c [lindex $chanlist $chanindex]} else {set c ""}
   if {$segindex >= 0}  then {set s [lindex $seglist  $segindex]}  else {set s ""}

   write-reg $c $s $reg $v 0
}

proc write-vxi-attrib {reg v} {
   global chanlist chanindex seglist segindex
    
   if {$chanindex >= 0} then {set c [lindex $chanlist $chanindex]} else {set c ""}
   if {$segindex >= 0}  then {set s [lindex $seglist  $segindex]}  else {set s ""}

   write-attribute $c $s $reg $v 0
}

proc write-common-reg {c reg v g} {write-reg $c "" $reg $v $g}

proc read-common-reg {chan reg} {read-reg $chan "" $reg}

proc read-reg {chan seg reg} {
   global VXIServer resource cardtype chantype segindex
   if {$resource != $VXIServer} then {set resource [VXImodule2crate $cardtype$chan]}
   if {$seg == "" || $seg == "nil"} then {
      set rr /$resource/$cardtype$chan.$reg
   } else {
      set rr /$resource/$cardtype$chan.$chantype$seg.$reg
   }
   EG "register :name '$rr' ?value"
}

proc read-attribute {chan seg reg} {
   global VXIServer resource cardtype chantype segindex
   if {$resource != $VXIServer} then {set resource [VXImodule2crate $cardtype$chan]}
   if {$seg == "" || $seg == "nil"} then {
      set rr /$resource/$cardtype$chan.$reg
   } else {
      set rr /$resource/$cardtype$chan.$chantype$seg.$reg
   }
   EG "register :name '$rr' ?object"
}

proc read-vxi-reg {reg} {
   global chanlist chanindex seglist segindex
    
   if {$chanindex >= 0} then {set c [lindex $chanlist $chanindex]} else {set c ""}
   if {$segindex >= 0}  then {set s [lindex $seglist  $segindex]}  else {set s ""}

   read-reg $c $s $reg
}

proc read-vxi-attrib {reg} {
   global chanlist chanindex seglist segindex
    
   if {$chanindex >= 0} then {set c [lindex $chanlist $chanindex]} else {set c ""}
   if {$segindex >= 0}  then {set s [lindex $seglist  $segindex]}  else {set s ""}

   read-attribute $c $s $reg
}

proc read-regs {chan seg reg} {
   global cardtype
   set rr [make-reg-name $chan $seg $reg]
   if {[llength [split "$rr" ?\[*]] == 1} then {
      set rr [make-reg-name $chan $seg $reg]
      set vl [EG "register :name '$rr' ?value"]
      set reg [last [split $rr /]]
      return [list "$reg $vl"]
   } else {
      EG "register :pattern '$rr'  ?names-and-values"
   }
}

proc read-vxi-regs {reg} {
   global chanlist chanindex seglist segindex
    
   if {$chanindex >= 0} then {set c [lindex $chanlist $chanindex]} else {set c ""}
   if {$segindex >= 0}  then {set s [lindex $seglist  $segindex]}  else {set s ""}

   read-regs $c $s $reg
}

proc init-vxi-reg {reg} {
   global chanlist chanindex seglist segindex
    
   if {$chanindex >= 0} then {set c [lindex $chanlist $chanindex]} else {set c ""}
   if {$segindex >= 0}  then {set s [lindex $seglist  $segindex]}  else {set s ""}

   set rr [make-reg-name $c $s $reg]
   EG "register :pattern '$rr' !initialise"
}

proc read-reg-names {} {
   global resource VXIServer cardtype chanlist chanindex
   if {$chanindex >= 0} then {set chan [lindex $chanlist $chanindex]} else {set chan ""}
   if {$resource != $VXIServer} then {set resource [VXImodule2crate $cardtype$chan]}
   set rr /$resource/$cardtype$chan.*
   EG "register :pattern '$rr' ?names"
}

#
#    procedure to determine the channels available for a card and to update the
#    choice menu
#


proc make-chanlist {} {
	global debug VXIServer cardtype defaultchanlist chanlist chanindex frameformat
        if {$debug} then {puts stderr "called proc make-chanlist"}

	   set chanlist {}

           if {$cardtype == "Trig" || $cardtype == "TriggerMK2"} then {
              set pattern '/$VXIServer/$cardtype.VXI.Passed'
           } else {
	      set pattern '/$VXIServer/$cardtype\[?,??,???\].VXI.Passed'
           }

	   if {![catch {eg "(register :pattern $pattern ?names)" } list]} then {
		foreach name $list {
                   if {[string last # $name] == -1} then {
                      if {$cardtype == "Trig" || $cardtype == "TriggerMK2"} then {
                         set chanlist {}
                         set chanindex -1
                         catch {:modify CHANNEL :strings [quote-list $chanlist] :value $chanindex}
                         return
                      } else {
			 scan $name [set cardtype]%d number
			 lappend chanlist $number
                      }
                   }
		}
	   }

        set chanlist [lsort -integer $chanlist]
	if {[llength $chanlist] == 0} then {
           if {![info exists defaultchanlist]} then {set chanlist ???} \
           else {set chanlist $defaultchanlist}
        }
	if {$chanlist == "???"} then {
           set chanindex -1
	} else {
	   if {$chanindex >= [llength $chanlist] || $chanindex < 0} then {
              set chanindex 0
           }
        }
        if {$frameformat == 1} then {
              catch {:modify CHANNEL :strings [quote-list $chanlist] :value $chanindex}
        }
}

proc make-myserver {} {
   global chanlist chanindex cardtype allchans
   global myserver resource VXIServer

if {$chanindex < 0} then {set myserver [VXImodule2crate $cardtype]} \
   else {set myserver [VXImodule2crate $cardtype[lindex $chanlist $chanindex]]}

  if {$allchans == 0} then {set resource $myserver} else {set resource $VXIServer}
}


#
#    various repaint procedures      ##############################################
#


proc repaint-widget widget {
   global  debug table frameformat
   if {$debug} then {puts stderr "proc repaint-widget called with argument $widget"}
   set line $table($widget)
   set param [lindex $line 0]
   set w2p [lindex $line 1]
   set p2w [lindex $line 2]
   set reg [lindex $line 3]
   set r2p [lindex $line 4]
   set p2r [lindex $line 5]
#	if {$frameformat==2 && $w2p=="slider-widget->param"} then {
#		set p2w param->str-widget}
   set unit [lindex $line 6]
   global $param
   set v [set $param]
   if {$debug} then {puts stderr "repaint-widget:  $p2w $widget $v $unit"}
      eval :modify [$p2w $widget $v $unit]
}
  
proc repaint-channel {} {
   global chanlist seglist chanindex segindex
      if {$segindex == -1} then {
         catch {:modify CHANNEL :value $chanindex}
      } else {
         catch {:modify CHANNEL :value $chanindex}
         catch {:modify SEGMENT :value $segindex}
         catch {:modify PARILCHANNEL :value "[lindex $chanlist $chanindex][lindex $seglist $segindex]"}
      }
}

proc repaint-global {} {
   global allchans allsegs segindex
     catch {:modify ALLCHANS :value $allchans}
     if {$segindex >= 0} then {
        catch {:modify ALLSEGS :value $allsegs}
     }
}

proc repaint-parameter {} {
   global register regnames
     catch {:modify PARAM :value [lsearch $regnames $register]}
}


#
#    procedures to extract the channel and segment from a register name    ########################
#

proc get-channel r {
   global cardtype
   set s [split $r "."]
   set s [string range $s [string length $cardtype] end]
   scan $s "%d" n
   return $n
}

proc get-segment r {
   global chantype
   set s [second [split $r "."]]
   if {[string range $s 0 [expr [string length $chantype] - 1]] == "$chantype"} then {
      set s [string range $s [string length $chantype] end]
   } else {
      set s -1
   }
   return $s
}

#
#    procedure to update widgets given  register name and value
#


if {[info exists table] == 1} then {
#    array table is defined

proc update-pair pair {
   global debug regnames widgetnames frameformat unshown
   global chanindex segindex

   if {$debug} then {insert-debug "proc update-pair called with argument $pair"}
   set r [strip-to-dot [first $pair]]
   set i [lsearch $regnames $r]
   if {$i<0} then {
      set r [strip-to-dot $r]
      set i [lsearch $regnames $r]
      if {$i<0} then {return}
   }
   if {$chanindex >= 0} then {set c [get-channel [first $pair]]} else {set c -1}
   if {$segindex >= 0}  then {set s [get-segment [first $pair]]} else {set s -1}
   set v [second $pair]
   if {$v==""} then {return}
   set i [lsearch $regnames $r]
   if {$i<0} then {return}
   set w [lindex $widgetnames $i]
   global table
   set line $table($w)
   set param [lindex $line 0]
   set w2p [lindex $line 1]
   set p2w [lindex $line 2]
   set reg [lindex $line 3]
   set r2p [lindex $line 4]
   set p2r [lindex $line 5]
   set unit [lindex $line 6]
#	if {$frameformat==2 && $w2p=="slider-widget->param"} then {
#		set p2w param->str-widget}
   if {$debug} then {insert-debug "update-pair executing $r2p $param $v"}
   $r2p $param $v
   global $param
   set v [set $param]
   if {$debug} then {insert-debug "update-pair:  $p2w $w $c $s $v $unit"}
   catch {
      switch $frameformat {
         1 {
              eval :modify [$p2w $w $v $unit]
         }
         2 {
             if {$s == -1} then {
                eval :modify [$p2w [set w]_[set c]_ $v $unit]
             } else {
                eval :modify [$p2w [set w]_[set c]_[set s] $v $unit]
             }
         }
      }
   }
}

}


if {[info exists table] == 0} then {
#    no array table is defined

proc update-pair pair {
    global debug wname r2v
    if {$debug} then {puts stderr "proc update-pair called with argument $pair"}

    set register [strip-to-dot [first $pair]]
    set value [second $pair]
    catch {
	set widget $wname($register)
	set val    [$r2v($widget) $value]
           :modify $widget :value $val
    }
    if {$register=="ADC"} then {
	catch {
              :modify ADC-TEXT :value $value
        }
    }
}

}

proc update-pairs l {foreach p $l {update-pair $p}}


####
#    common (or default) click procedures       ####################################
####

#    first the default click procedure

proc click-on {ww v} {
   global debug table chanlist chanindex segindex seglist frameformat allchans allsegs
   if {$debug} then {puts stderr "proc click-on called with arguments \{$ww\} \{$v\}"}

   switch $frameformat {
      1  {
         set g 0
         set w $ww
         if {$chanindex >= 0} then {
            set c [lindex $chanlist $chanindex]
            set g 1
         } else {
            set c ""
         }
         if {$segindex >= 0} then {
            set s [lindex $seglist $segindex]
            set g 1
         } else {
            set s ""
         }
      }
      2  {
          set g 1
          set w [first [split $ww "_"]]
          set c [second [split $ww "_"]]
          if {$segindex >= 0} then {
            set s [third [split $ww "_"]]
          } else {
            set s ""
          }
      }
   }
   set line "$table($w)"
   set param [lindex $line 0]
   set w2p [lindex $line 1]
   set p2w [lindex $line 2]
   set reg [lindex $line 3]
   set r2p [lindex $line 4]
   set p2r [lindex $line 5]
   set unit [lindex $line 6]
   set step [lindex $line 10]
   if {$debug} then {puts stderr "proc click-on executing  \{$w2p $param $v $step\}"}
   $w2p $param $v $step
   if {$reg!="nil"} then {
      write-reg $c $s $reg [$p2r $param $v $unit] $g
	if {$frameformat==2} then {
	    if {$allchans || $allsegs} then {click-redisplay}
	} else {
	    update-pairs [read-regs $c $s $reg]
	}
   }
   if {$reg!="nil" && $allchans} then {set allchans 0; make-myserver; repaint-global}
}

  
proc click-channel v {
   global debug chanindex
   if {$debug} then {puts stderr "proc click-channel called with argument \{$v\}"}
   set chanindex $v
   make-myserver
   click-redisplay
}

proc click-segment v {
   global debug segindex
   if {$debug} then {puts stderr "proc click-segment called with argument \{$v\}"}
   set segindex $v
   click-redisplay
}

proc click-param v {
   global debug widgetnames regnames register parameter
   if {$debug} then {puts stderr "proc click-param called with argument \{$v\}"}
   set w [lindex $widgetnames $v]
   set parameter $w
   set register [lindex $regnames $v]
   make-panel $parameter
   click-redisplay
}


proc click-electronics {} {
#  display electronics sub frame
   global electronicsvisible electronicsframe
   if {$electronicsvisible==0} then {make-electronics} else {open-frame $electronicsframe}
   click-redisplay
}
  
proc click-software {} {
   global cardtype
   inform [string tolower $cardtype]sftwre start
}
  

proc click-format {} {
   global debug frameformat frame parameter
   global electronicsvisible  girvisible
   if {$debug} then {puts stderr "proc click-format called for format $frameformat"}
   if {$frameformat==1} then {
	set electronicsvisible 0
        set girvisible 0
	set frameformat 2
        :modify $frame :delete T
        make-frame2
	make-panel $parameter
   } else {
	set electronicsvisible 0
        set girvisible 0
	set frameformat 1
        :modify $frame :delete T
        make-frame1
#	repaint-presets-menu
   }
   click-redisplay
}

proc click-redisplay {} {
   global frameformat cardtype chanlist chanindex segindex seglist
   global RegsData unshown
   global register allchannels allsegments
   global suppressionvisible
   global inspection
   global resource myserver VXIServer

   if {![:query_Mapped .]} then {return}

   set resource $myserver

   if {$cardtype == "VRE"} then  {fw-paint-directory}
   repaint-channel
   switch $frameformat {
      1  {
         switch $cardtype {
            Trig        -
            Ge          -
            BGO         -
            BgoBall     -
            VRE         -
            SlotOne     -
            RM          {redisplay *}
            default     {Redisplay}
         }
      }
      2 {
        set resource $VXIServer
        set RegsData [read-regs $allchannels $allsegments $register]
        set resource $myserver
        foreach pair $RegsData {
            if {[llength [split [first $pair] #]] == 1} then {update-pair $pair}
        }
      }
   }
   update-inspection
   set unshown 0

   if {$frameformat == 1} then {act_on_tfagain}

   repaint-global
   repaint-parameter
   if {$suppressionvisible} then {suppression-redisplay}

   make-myserver
} 


proc Redisplay {} {
  global RegsData VXIGlobal EGallow EGreport 
  global resource table cardtype chantype chanlist chanindex seglist segindex

  set EGallow_BAK $EGallow
  set EGallow 0x10004

  set EGreport_BAK $EGreport
  set EGreport -1
  set-busy

  set chan [lindex $chanlist $chanindex]
  if {$segindex >= 0} then {set seg [lindex $seglist $segindex]}

  set RegsData {}

  foreach widget [array names table] {
     set entry $table($widget)
     set reg [lindex $entry 3]

    if {$reg == "nil"} then {continue}

     if {$segindex >= 0} then {

          set rr  /$resource/$cardtype$chan.$chantype$seg.$reg
          set data [EG "register :name '$rr' ?value"]
          if {$data == $EGallow} then {
              if {$chantype == ""} then {
                  set rr  /$resource/$cardtype$chan.$reg
              } else {
                  set rr  /$resource/$cardtype$chan.$chantype.$reg
              }
              set data [EG "register :name '$rr' ?value"]
              if {$data == $EGallow && $chantype != ""} then {
                  set rr  /$resource/$cardtype$chan.$reg
                  set data [EG "register :name '$rr' ?value"]
              }
          }
     } else {

          if {$chantype == ""} then {
              set rr  /$resource/$cardtype$chan.$reg
          } else {
              set rr  /$resource/$cardtype$chan.$chantype.$reg
          }
          set data [EG "register :name '$rr' ?value"]
          if {$data == $EGallow && $chantype != ""} then {
             set rr  /$resource/$cardtype$chan.$reg
             set data [EG "register :name '$rr' ?value"]
          }

     }
     lappend RegsData "$widget $rr $data"
     if {$data != $EGallow} then {Update-Widget $widget $data}
  }

  clear-busy
  set EGallow $EGallow_BAK
  set EGreport $EGreport_BAK
}

#    new version of update-pair used by proc Redisplay
#    first argument the a widget name
#    second argument is the corresponding value

proc Update-Widget {w value} {
   global debug frameformat
   global chanlist seglist chanindex segindex

   if {$debug} then {insert-debug "proc Update-Widget called with arguments $w $value"}

   global table
   set line $table($w)
   set param [lindex $line 0]
   set p2w   [lindex $line 2]
   set r2p   [lindex $line 4]
   set unit  [lindex $line 6]

   if {$debug} then {insert-debug "Update-Widget executing $r2p $param $value"}

   $r2p $param $value
   global $param
   set v [set $param]

#    now update the widget
   catch {
      switch $frameformat {
         1 {
              if {$debug} then {insert-debug "Update-Widget:  $p2w $w $v $unit"}
              eval :modify [$p2w $w $v $unit]
         }
         2 {
             if {$chanindex >= 0} then {set c [lindex $chanlist $chanindex]} else {set c -1}
             if {$segindex >= 0}  then {set s [lindex $seglist $segindex]}   else {set s -1}
             if {$s == -1} then {
                if {$debug} then {insert-debug "Update-Widget:  $p2w $w $c $v $unit"}
                eval :modify [$p2w [set w]_[set c]_ $v $unit]
             } else {
                if {$debug} then {insert-debug "Update-Widget:  $p2w $w $c $s $v $unit"}
                eval :modify [$p2w [set w]_[set c]_[set s] $v $unit]
             }
         }
      }
   }
}

proc act_on_tfagain {} {
  global tfagain table CFDTHRESHLOW CFDTHRESHHIGH

#    check if this module actually has a TFA gain adjustment
#    if it does then the range of the CFD Threshold slider has
#    to be set according to the TFA gain range

  if {[info exists tfagain] == 0} then {return}

  if {$tfagain == 0} then {
#    low gain setting
     catch {:modify CFDTHRESH :max 2600}
     set table(CFDTHRESH) $CFDTHRESHLOW
  } else {
     catch {:modify CFDTHRESH :max 825}
     set table(CFDTHRESH) $CFDTHRESHHIGH
  }
}


#    procedures to support the channel and segment selection facility
#    note at present this requires a special version of the write-reg procedure

proc click-whichchans v {
  global allchans chanindex chansel
  switch $v {
    0  -
    1  {set allchans $v}
    2  {
       set allchans -1
       if {$chansel == 0} then {set chansel [expr 1 << $chanindex]}
       make-chanSELframe
    }
  }
  make-myserver
}

proc click-whichsegs v {
  global allsegs segindex segsel
  switch $v {
    0  -
    1  {set allsegs $v}
    2  {
       set allsegs -1
       if {$segsel == 0} then {set segsel [expr 1 << $segindex]}
       make-segSELframe
    }
  }
}

proc click-chanselection v {
  global chansel chanlist chanselection
  set chansel $v
  set chanselection "\["
  foreach i [list-take $chanlist [bits-in $chansel]] {
    append  chanselection $i,
  }
  append chanselection "\]"
}

proc click-segselection v {
  global segsel seglist segselection
  set segsel $v
  set segselection "\["
  foreach i [list-take $seglist [bits-in $segsel]] {
    append segselection  $i,
  }
  append segselection "\]"
}

#   procedure to update widget corresponding to a register or group of registers

proc redisplay {arg} {
	global debug RegsData resource chanlist chanindex seglist segindex
        if {$debug} then {puts stderr "called proc redisplay with argument \{$arg\}"}

        if {$segindex >= 0} then {
	   set RegsData [read-regs [lindex $chanlist $chanindex] [lindex $seglist $segindex] $arg]
        } else {
	   set RegsData [read-regs [lindex $chanlist $chanindex] "" $arg]
        }
	update-pairs $RegsData
}

proc redisplay-widget-reg widget {
   global table
   set line $table($widget)
   set reg [lindex $line 3]
   redisplay $reg
}

proc write-widget-reg {ww v} {
   global table chanlist chanindex segindex seglist 

   set g 0
   set w $ww
   if {$chanindex >= 0} then {
     set c [lindex $chanlist $chanindex]
     set g 1
   } else {
     set c ""
   }
   if {$segindex >= 0} then {
     set s [lindex $seglist $segindex]
     set g 1
   } else {
     set s ""
   }
 
   set line "$table($w)"
   set param [lindex $line 0]
   set w2p [lindex $line 1]
   set p2w [lindex $line 2]
   set reg [lindex $line 3]
   set r2p [lindex $line 4]
   set p2r [lindex $line 5]
   set unit [lindex $line 6]
   set step [lindex $line 10]
   $w2p $param $v $step

   write-reg $c $s $reg [$p2r $param $v $unit] $g
}

proc read-widget-reg {w} {
   global table chanlist chanindex segindex seglist 

   if {$chanindex >= 0} then {
     set c [lindex $chanlist $chanindex]
   } else {
     set c ""
   }
   if {$segindex >= 0} then {
     set s [lindex $seglist $segindex]
   } else {
     set s ""
   }
 
   set line "$table($w)"
   set reg [lindex $line 3]

   read-reg $c $s $reg 
}




######
#    procedures to handle inspection tables and presets             ###########################
######


proc repaint-presets-menu {} {
   global inspection frameformat presetslist

   if {$inspection == 0} then {return}
   if {$frameformat != 1} then {return}

   set presetsmenu $presetslist
   lappend presetsmenu "" "" "Grab Setting" "Drop Setting" "Disconnect All"
#   :modify PRESETS-MENU :strings [quote-list $presetsmenu]
}

proc make-presets-pattern {} {
   global debug table chanlist chanindex seglist segindex
   global inspection_regnames inspection_widgetnames
   global analogueinsptable digitalinsptable voltageinsptable
   if {$debug} then {puts stderr "proc make-presets-pattern called"}

   set pattern {}
   for {set i 0} {$i < 5} {incr i} {
     set c [read-common-reg [lindex $chanlist $chanindex] [lindex $inspection_regnames $i].Channel]
     set w [lindex $inspection_widgetnames $i]
     if {$c == 0} then {set s 0} else {
       if {$c < 9 && [expr $c - 1] != $segindex} then {set s 0} else {
         set s [read-common-reg [lindex $chanlist $chanindex] [lindex $inspection_regnames $i].Signal]
         set line "$table($w)"
         set f [lindex $line 1]
         set l [lindex $line 2]
         set o [lindex $line 3]
         set s [expr $s + 1 - $f - $o] 
       }
     }

    if {$i == 0 || $i == 1} then {set ip [lindex $analogueinsptable $s]} else {
    if {$i == 2 || $i == 3} then {set ip [lindex $digitalinsptable $s]} else \
    {set ip [lindex $voltageinsptable $s]}}
    if {$ip == "Disconnected"} then {set ip ""}
    lappend pattern "$ip"
  }
  set pattern [drop-trailing-nulls $pattern]
  set pattern [join $pattern "/"]   
  return $pattern
}   


proc click-presets-grab {} {
   global debug presetslist
   if {$debug} then {puts stderr "proc click-presets-grab called"}
   set pattern [make-presets-pattern]
   if {$pattern!=""} then {lappend presetslist $pattern}
   repaint-presets-menu
}

proc click-presets-drop {} {
   global debug presetslist
   if {$debug} then {puts stderr "proc click-presets-drop called"}
   set pattern [make-presets-pattern]
   set presetslist [list-drop $presetslist $pattern]
   repaint-presets-menu
}

proc list-drop {ss p} {
   set n [lsearch $ss $p]
   if {$n<0} then {return $ss}
   lreplace $ss $n $n
}

proc click-presets-disconnect-all {} {
   global debug
   if {$debug} then {puts stderr "proc click-presets-disconnect-all called"}
   click-presets-set "Disconnected/Disconnected/Disconnected/Disconnected/Disconnected"
}

proc click-presets-set v {
   global debug inspection_widgetnames
   global analogueinsptable digitalinsptable voltageinsptable
   if {$debug} then {puts stderr "proc click-presets-set called with argument \{$v\}"}

   append items $v "/////"
   set items [split $items "/"]
   for {set i 0} {$i < 5} {incr i} {
     set item [lindex $items $i]
     set item [split $item " "]
     set item [first $item]
     if {$item != ""} then {
       set w [lindex $inspection_widgetnames $i]
       if {$i == 0 || $i == 1} then {set s [lsearch $analogueinsptable $item]} else {
       if {$i == 2 || $i == 3} then {set s [lsearch $digitalinsptable $item]} else \
       {set s [lsearch $voltageinsptable $item]}}
       click-inspection $w $s
     }
   }
   update-inspection
}

proc click-presets v {
   global debug chanlist chanindex redisplay
   if {$debug} then {puts stderr "proc click-presets called with argument \{$v\}"}

   switch $v {
     {Disconnect All } {click-presets-disconnect-all}
     {Grab Setting }   {click-presets-grab}
     {Drop Setting }   {click-presets-drop}
     default           {click-presets-set $v}
   }
}


#
#    procedures to handle inspection line displays  
#


proc click-inspection {w v} {
   global debug cardtype chantype insp_table chanlist chanindex seglist segindex
   global analogueinsptable digitalinsptable voltageinsptable parilinsptable
   if {$debug} then {puts stderr "proc click-inspection called with arguments \{$w\} \{$v\}"}
#   insert-debug "proc click-inspection called with arguments \{$w\} \{$v\}"

   set c [lindex $chanlist $chanindex]
   if {$segindex >= 0} then {set s [lindex $seglist $segindex]} \
      else {set s ""}

  switch $cardtype {
  
   ADC6414V     -
   Icare        -
   Demon        -
   Saphir       -
   CsI          -
   HRGe8x       -
   ECC          -
   ESS          -
   GOCCE        -
   TDR32V        -
   TGIR         -
   FVI          -
   TriggerMK2   -
   ClusterGe    -
   ClusterBGO   {

#    first all new style (GIR) modules and modules using the new form of
#    software setup and control

   set line "$insp_table($w)"
   set reg [lindex $line 0]
   set inspection [lindex $line 1]

#    via the xMux class register select the channel

   if {$reg != "PARIL.Insp1" && $reg != "PARIL.Insp2"} then {
      if {$cardtype == "TDR32V" && $inspection == "DI"} then {
         global [set inspection]-ChipSelect
         set csr [lindex [set [set inspection]-ChipSelect] $v]
      } else {
         global [set inspection]-ChanSelect
         set csr [lindex [set [set inspection]-ChanSelect] $v]
      } 

#    check for value "*" which implies value is determined by adc channel

         switch $cardtype {
           Icare          {if {$csr == "*"} then {set csr [expr $chanindex - [expr $chanindex / 10 * 10]]}}
           Demon          {if {$csr == "*"} then {set csr [expr 1+ $chanindex - [expr $chanindex / 6 * 6]]}}
           Saphir         {if {$csr == "*"} then {set csr [expr $chanindex - [expr $chanindex / 16 * 16] + 1]}}
           CsI            {if {$csr == "*"} then {set csr [expr $chanindex - [expr $chanindex / 8 * 8] + 1]}}
           HRGe8x         {if {$csr == "*"} then {set csr [expr (($chanindex - ($chanindex / 2 * 2)) * 4) + $segindex +1]}}
           ECC            {if {$csr == "*"} then {set csr [expr (($chanindex - ($chanindex / 2 * 2)) * 4) + $segindex +1]}}
           ESS            {if {$csr == "*"} then {set csr [expr $chanindex - [expr $chanindex / 4 * 4] + 1]}}
           GOCCE          {if {$csr == "*"} then {set csr [expr $segindex + 1]}}
           TDR32V  -
           ClusterBGO     {if {$csr == "*"} then {set csr $segindex}}
           ClusterGe      {if {$csr == "*"} then {set csr [expr $segindex + 1]}}
           default        {if {$csr == "*"} then {set csr 0}}
         }

      if {$cardtype == "TDR32V" && $inspection == "DI"} then {
          write-insp-reg $c $s $reg.Chip $csr 0
      } else {
          write-insp-reg $c $s $reg.Channel $csr 0
      }
   }

   if {$cardtype == "TDR32V" && $inspection == "DI"} then {

#    via the xDig class register select the channel

       global [set inspection]-Select
       set csr  [lindex [set [set inspection]-Select] $v]

#    check for the value "*" which implies use the ADC (segment) number

       if {$csr == "*"} then {set csr $segindex}
       write-insp-reg $c $s $reg.Select $csr 0
   }

#    via the xDig class register select the signal

   global [set inspection]-Param
   set pr  [lindex [set [set inspection]-Param] $v]

#    check for value "*" which implies disconnect

   if {$pr == "*"} then {
      set pr 0
      set action "Disconnected"
      switch $cardtype {
         Saphir      -
         CsI         -
         FVI         -
         TriggerMK2  -
         TDR32V      -
         ClusterBGO  {set register Signal}
         default     {set register Channel}
      }
      if {$cardtype == "TDR32V" && $inspection == "DI"} then {set register Chip}
   } else {
      switch -glob $w {
         MUX-AI*   {set signal [lindex $analogueinsptable $v]}
         MUX-DI*   {set signal [lindex $digitalinsptable $v]}
         MUX-VI    {set signal [lindex $voltageinsptable $v]}
         MUX-PAR*  {set signal [lindex $parilinsptable $v]}
      }
      set action "Connected to $cardtype$c$s signal $signal"
      switch $cardtype {
         default     {set register Signal}
      }
   }

      if {$reg != "PARIL.Insp1" && $reg != "PARIL.Insp2"} then {
         write-insp-reg $c $s $reg.$register $pr 0
      } else {
         write-reg $c $s $reg $pr 0
      }
   }
   default  {

#    default is all old style (Eurogam) modules

#      click-on $w $v
      switch -glob $w {
         MUX-AI*   {set signal [lindex $analogueinsptable $v]}
         MUX-DI*   {set signal [lindex $digitalinsptable $v]}
         MUX-VI    {set signal [lindex $voltageinsptable $v]}
      }

      set line "$insp_table($w)"
      set reg [lindex $line 0]

      write-reg $c $s $reg $signal 0

      if {$signal == "Disconnected"} then {set action "Disconnected"} \
         else {set action "Connected to $cardtype$c$chantype$s signal $signal"}
   }

  }

#    record Inspection Line use

  set line [second [split $w -]]
  set crate [VXImodule2crate $cardtype$c]
  EG "register :name '/$crate/NetVar_Inspection_$line'  :value '$action'"

  if {"$action" == "Disconnected"} then {:modify [set w]-USE :label "Disconnected"} \
      else {:modify [set w]-USE :label "$signal"}

#    finally connect to inspection line router if present

  if {"$action" != "Disconnected"} then {connect-inspection $line $crate}
}


set vxicontrolstarted 0

proc VXImodule2crate {module} {
  global vxicontrolstarted

#    obtain crate holding a specific module
#    this uses the task  "vxicontrol"

  if {$vxicontrolstarted == 0} then {
     set started [inform vxicontrol inquire started]
     if {$started == 0} then {inform vxicontrol make-frame}
     set vxicontrolstarted 1
  }

  set crate [send vxicontrol VXImodule2crate $module]
  return $crate
}

proc update-inspection {} {
   global cardtype chantype chanlist chanindex seglist segindex myserver
   global inspection inspection_widgetnames
   global frameformat

   if {$inspection == 0} then {return}
   if {$frameformat != 1} then {return}

   set c [lindex $chanlist $chanindex]
   if {$segindex >= 0} then {set s [lindex $seglist $segindex]} \
      else {set s ""}

   set crate $myserver
   set inspallocn [EG "register :pattern '/$crate/NetVar_Inspection_\[????,???,??\]'  ?names-and-values"]

   foreach item $inspallocn {

        set w MUX-[third [split [first $item] _]]
        set allocn [second $item]

        if {"[first $allocn]" == "Connected"} then {
            if {"[third $allocn]" == "$cardtype$c$s" || "[third $allocn]" == "$cardtype$c$chantype$s"} then {
#    this inspection line is in use by this window - output only signal name
                catch {:modify [set w]-USE :label "[lrange $allocn 4 end]"}
            } else {
#    this inspection line is in use by something else - output full connection information
                catch {:modify [set w]-USE :label "$allocn"}
            } 
        } else {
           catch {:modify [set w]-USE :label "Disconnected"}
        }
   }
}

#
####  
#   procedures to handle the channel step facility
####
#

set stepmenu [list "forward" "back" "step only" "step + settings" "step + inspection" "step + both" "set + insp & enable"]
set stepmode 4

proc click-step {v i} {
  global stepmode stepmenu
  switch $i {
     0           {do-step 1; return}
     1           {do-step -1; return}
     default     {set stepmode $i}
  }
  :modify STEP :label "$v"
}


proc do-step {v} {
  global stepmode stepmenu  table segindex seglist chanindex chanlist
  global EGreport myserver VXIGlobal
  global cardtype chantype insp_menu insp_table
  global analogueinsptable digitalinsptable voltageinsptable
  global stepmenu stepmode
  global steplist1 steplist2

  set EGreport_save $EGreport
  if {$EGreport == 0} then {set EGreport -1}
  set-footer "Stepping in mode [lindex $stepmenu $stepmode]"
  set-busy
   
#    work out the full register name prefixes for registers for current channel
#    and segment
   if {$chanindex >= 0} then {set c [lindex $chanlist $chanindex]} else {set c ""}
   if {$segindex >= 0}  then {set s [lindex $seglist  $segindex]}  else {set s ""}

  if {$stepmode == 3 || $stepmode == 5} then {
#    modes "step + settings" and "step + both"
#    need to save settings required later
#    obtain current values for the registers associated with widgets in "steplist"

     set savelist {}
     foreach item $steplist1 {
         set rr /$myserver/$cardtype$c.$VXIGlobal.[lindex $table($item) 3]
         set vl [EG "register :name '$rr' ?value"]
         lappend savelist "[lindex $table($item) 3]  $vl"
     }

     foreach item $steplist2 {
         set rr /$myserver/$cardtype$c.$chantype$s.[lindex $table($item) 3]
         set vl [EG "register :name '$rr' ?value"]
         lappend savelist "[lindex $table($item) 3]  $vl"
     }
  }
  if {$stepmode == 4 || $stepmode == 5 || $stepmode == 6} then {
#    modes "step + inspection" and "step + both"
#    need current inspection line settings

   switch $cardtype {
     Saphir     -
     CsI        -
     HRGe8x     -
     ECC        -
     ESS        -
     TDR32V      -
     GOCCE      -
     ClusterGe  -
     ClusterBGO  {set prefix1 $cardtype$c$s}
     default     {set prefix1 $cardtype$c$chantype$s}
   }
   if {$stepmode == 6} then {
#    disable current channel
        set rr /$myserver/$cardtype$c.$chantype$s.[lindex $table(ENABLE) 3]
        EG "register :name '$rr' :value '0'"
   }

    set inspallocn [EG "register :pattern '/$myserver/NetVar_Inspection_\[???,??\]'  ?names-and-values"]
    set inspsavelist {}

    foreach item $inspallocn {

        set line [third [split [first $item] _]]
        set allocn [second $item]

        if {"[first $allocn]" == "Connected"} then {
            if {"[third $allocn]" == "$prefix1"} then {
#    this inspection line is in use by this window - 
                lappend inspsavelist [list $line "[fifth $allocn]"]
            }
        }
     }
  }

#    first the simple job - step the segment/channel
  if {$segindex >= 0} then {
     incr segindex $v
     if {$segindex == [llength $seglist]} then {
       set segindex 0
       incr chanindex
       if {$chanindex == [llength $chanlist]} then {set chanindex 0}
     }
     if {$segindex < 0} then {
       set segindex [expr [llength $seglist] - 1]
       incr chanindex $v
       if {$chanindex < 0} then {set chanindex [expr [llength $chanlist] - 1]}
     }
  } else {
     incr chanindex $v
     if {$chanindex == [llength $chanlist]} then {set chanindex 0}
     if {$chanindex < 0} then {set chanindex [expr [llength $chanlist] - 1]}
  }

   if {$chanindex >= 0} then {set c [lindex $chanlist $chanindex]} else {set c ""}
   if {$segindex >= 0}  then {set s [lindex $seglist  $segindex]}  else {set s ""}

  make-myserver
  if {$stepmode == 3 || $stepmode == 5} then {
#    modes "step + settings" and "step + both"
#    load the saved settings into the new channel

     set indx 0
     foreach item $steplist1 {
         set rr /$myserver/$cardtype$c.$VXIGlobal.[lindex $table($item) 3]
         set vl [second [lindex $savelist $indx]]
         EG "register :name '$rr' :value '$vl'"
         incr indx
     }

     foreach item $steplist2 {
         set rr /$myserver/$cardtype$c.$chantype$s.[lindex $table($item) 3]
         set vl [second [lindex $savelist $indx]]
         EG "register :name '$rr' :value '$vl'"
         incr indx
     }
  }
  if {$stepmode == 4 || $stepmode == 5 || $stepmode == 6} then {
#    modes "step + inspection" and "step + both"
#    load the saved settings into the new channel
     foreach item $inspsavelist {
        set line MUX-[first $item]
        set setting [lsearch [set $insp_menu($line)] [second $item]]
        click-inspection $line $setting
     }
  }
   if {$stepmode == 6} then {
#    enable current channel
        set rr /$myserver/$cardtype$c.$chantype$s.[lindex $table(ENABLE) 3]
        EG "register :name '$rr' :value '1'"
   }
  click-redisplay
  set EGreport $EGreport_save
  clear-busy
  clear-footer
}

 
######
#    procedures to support parameter save/restore
######

set registerstosave ""

proc Return_Save_Registers {} {
   global registerstosave registersavepatterns
   global started allchans allsegs segindex EGreport
   if {$registerstosave == ""} then {
#    get list of registers available if not already done
      if {$started == 0} then {
         set allchans 0
         if {$segindex >= 0} then {set allsegs 0}
         make-chanlist
         make-myserver
         set EGreport -1
         set names [read-reg-names]
         set EGreport 0
      } else {set names [read-reg-names]}

#    remove the module component
      set available ""
      foreach name $names {
        set m [first [split $name .]]
        append available "[string range $name [expr [string length $m] + 1] end] "
      }

#    select which we need using the save patterns
      set registerstosave ""
      foreach item $registersavepatterns {
         foreach m $available {
            if {[string match $item $m]} then {append registerstosave "$m "}
         }
      }
   }
   return $registerstosave
}


######
#    procedures to handle suppression tables            #########################
######


proc click-suppression v {
	global debug
        global suppressiontable suppressionvisible suppressionframe
        global chanlist chanindex allchans frameformat

	if {$debug} then {puts stderr "click-suppression $v"}
	set value [lindex $suppressiontable $v]

	if {$value=="File..."} then {
		set reg "nil"
		if {$suppressionvisible==0} then {make-suppression} \
		    else {open-frame $suppressionframe}
		set suppressionvisible 1
	} else {
		set c [lindex $chanlist $chanindex]
		set reg BGO.SuppressionTable
		write-reg $c nil $reg $value $allchans
		if {$allchans && $frameformat==2} then {
		   click-redisplay
		} else {
		   update-pair [read-reg $c "" $reg]
		}
	}
	if {$suppressionvisible} then {suppression-redisplay}
 if {$reg != "nil" && $allchans} then {set allchans 0; make-myserver; repaint-global}
}
 

proc click-suppression-load {} {
   global chanlist chanindex suppressionfile suppressiondirectory allchans

   set c [lindex $chanlist $chanindex]
   set reg BGO.SuppressionTable
   write-reg $c nil $reg $suppressiondirectory/$suppressionfile $allchans
   click-redisplay
   if {$allchans} then {set allchans 0; make-myserver; repaint-global}
}

proc suppression-redisplay {} {
   global debug chanlist chanindex suppressiondirectory suppressionfile
   suppression-repaint
   suppression-paint-directory
   set c [lindex $chanlist $chanindex]
   set reg BGO.SuppressionTable
   :modify SUP-CURRENT :value "[read-reg $c nil $reg]"
}

proc click-suppression-file name {
   global suppressiondirectory suppressionfile 
   set suppressionfile $name
   suppression-repaint          
   if {$name != "" && [file isdirectory $suppressiondirectory/$name]} then {
      click-suppression-directory $suppressiondirectory/$name
   }
}

proc click-suppression-directory name {
   global suppressiondirectory suppressionfile
   set suppressiondirectory $name
   suppression-paint-directory
   set suppressionfile ""
   suppression-repaint
}

proc suppression-paint-directory {} {
   global suppressiondirectory
   set m [EG "directory :path '$suppressiondirectory' ?files"]
   set dirlist [lsort [strip-dots [make-list $m]]]
        :modify SUP-LIST :empty T :strings [make-list $dirlist]
}
                 
proc suppression-repaint {} {
   global suppressiondirectory suppressionfile 
     :modify SUP-DIRECTORY :value "$suppressiondirectory"
     :modify SUP-FILE      :value "$suppressionfile"
}

proc click-suppression-list {c file} {
   global suppressionfile
   switch $c {
      :list-select {click-suppression-file $file}
      :list-deselect {click-suppression-file ""}
   }
}

########
#    procedures to handle general firmware files  - used for LCA (trigger) and DSP (VRE)  ########
########

proc click-fw-dirname name {
   global fwdirname fwprogname
   set fwdirname $name
   set fwprogname ""
   fw-paint-directory
   fw-repaint
}

proc click-fw-progname name {
   global fwdirname fwprogname 
   set fwprogname $name
   fw-repaint          
   if {$name != "" && [file isdirectory $fwdirname/$name]} then {
      click-fw-dirname $fwdirname/$name
   }
} 

proc click-fw-list {c file} {

   switch $c {
      :list-select   {click-fw-progname $file}
      :list-deselect {click-fw-progname ""}
   }
}
            
proc fw-repaint {} {
   global fwdirname fwprogname 
     :modify FWDIR  :value "$fwdirname"
     :modify FWPROG :value "$fwprogname"
}

proc fw-paint-directory {} {
   global fwdirname
   set m [EG "directory :path '$fwdirname' ?files"]
   set dirlist [lsort [strip-dots [make-list $m]]]
   :modify FWLIST :empty T :strings [make-list $dirlist]
}


######
#    standard procedures for task  start and end                     ###########################
######

proc start args {
   global frameformat allchans allsegs segindex started
   global chanindex myserver cardtype chanlist chanindex
   if {$started != 0} then {open-frame} else {
	set allchans 0
        if {$segindex >= 0} then {set allsegs 0}
        set frameformat 1
	make-frame1
        EGbindings
        set started 1
   }
   set-busy
   make-chanlist
   make-myserver
   click-redisplay
   clear-busy
}

proc set-experiment args {
#    experiment has been joined (or changed)
   start
}

#    require slider to be handled with resolution of 256 steps
set _default_resolution 0

#    VXI tasks need vxicontrol
inform vxicontrol autostart


#    connected to Shift+F3

proc Xmidas-show-object {W} {
   global rname table
   set w [:widget $W]
   set l "This graphical object has path $W\n"
   append l "It is a component of widget $w\n"
   if {[info exists rname($w)]} then {
      append l "It is linked to Midas object $rname($w)\n"
      set info [read-vxi-attrib $rname($w)]
      append l "Object attributes are :-\n"
      append l "$info\n"
   }
   if {[info exists table($w)]} then {
      set entry $table($w)
      append l "It is linked to Midas object [second $entry]\n"
      set info [read-vxi-attrib [second $entry]]
      append l "Object attributes are :-\n"
      append l "$info\n"
   }
   midas-report $l
}
