# saphir.tcl---Setup frame for Saphir card

#     created 28/5/97  from Ge card frame   VP

set debug 0
set started 0
set frame SAPHIR
set frametitle "Saphir Card Setup"
set icontitle Saphir

set chanlist {???}
set chanindex 0

set defaultchanlist {0 1 2 3 4 5 6 7}

set cardtype Saphir
set VXIGlobal ""

set presetslist {}


#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"


set table(CHANNEL) {channel widget->channel channel->widget nil
        nil nil nil nil nil nil nil 1 nil}
set table(ALLCHANS) {allchans int-widget->param param->int-widget nil
        nil nil nil nil nil nil nil 0 nil}
       
set table(ENABLE) {enable int-widget->param param->int-widget "Enable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 0 "Enable"}
set table(AMP) {amp int-widget->param param->int-widget "CCR.Amp"
       int-reg->param param->int-reg nil :choice Int Ext nil 0 "External Amp"}
set table(INVERT) {invert int-widget->param param->int-widget "CCR.InvertPDS"
       int-reg->param param->int-reg nil :choice Off On nil 0 "PDS Invert"}
set table(CTESTGEN) {ctestgen int-widget->param param->int-widget "CCR.TestGen"
       int-reg->param param->int-reg nil :choice Off On nil 0 "Channel Test Generator"}
       
set table(MARKREJECT) {markreject int-widget->param param->int-widget "PileupReject"
       int-reg->param param->int-reg nil :choice Mark Reject nil 1 "Pileup"}

set table(CFDTHRESH) {cfdthresh int-widget->param param->int-widget "CFD.Thresh"
       int-reg->param param->int-reg nil :text 4 nil nil 1 "CFD Threshold"}
set table(CFDDELAY) {cfddelay slider-widget->param param->int-widget "CFD.Delay"
       unit-reg->param param->unit-reg ns :slider 0 200 1 1 "CFD Delay (ns)"}
set table(CFDWIDTH) {cfdwidth slider-widget->param param->int-widget "CFD.Width"
       unit-reg->param param->unit-reg ns :slider 0 200 1 1 "CFD Width (ns)"}

set table(FTSAMPLE) { ftsample slider-widget->param param->int-widget "FTSample"
       unit-reg->param param->unit-reg ns :slider 0 2000 8 1 "Fast Trigger Sample (ns)"}
set table(VALSAMPLE) {valsample slider-widget->param param->int-widget "ValSample"
       unit-reg->param param->unit-reg ns :slider 0 10000 40 1 "Validation Sample (ns)"}
set table(PDSGATE) {pdsgate slider-widget->param param->int-widget "PDSGate"
       unit-reg->param param->unit-reg ns :slider 0 10000 40 1 "PDS Gate (ns)"}  
       
set table(ADDRE) {addre widget->param param->hex-widget "Energy.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "Energy Address"}
set table(SSE) {sse int-widget->param param->int-widget "Energy.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Energy Sliding Scale"}
set table(READOUTE) {readoute int-widget->param param->int-widget "Energy.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Energy Readout"}

set table(ADDRT) {addrt widget->param param->hex-widget "Timing.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "Timing Address"}
set table(SST) {sst int-widget->param param->int-widget "Timing.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Timing Sliding Scale"}
set table(READOUTT) {readoutt int-widget->param param->int-widget "Timing.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Timing Readout"}

#    These are used by the electronics window
set table(MODELCODE) {modcode str-widget->param param->hex-widget "VXI.ModelCode"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(SERIALNUM) {serialnum str-widget->param param->hex-widget "VXI.SerialNum"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MODLEVEL) {modlevel str-widget->param param->hex-widget "VXI.ModLevel"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MANID) {manid str-widget->param param->hex-widget "VXI.ManID"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(OFFSET) {offset str-widget->param param->hex-widget "VXI.Offset"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(TESTGEN) {testgen slider-widget->param param->int-widget "TestGenAmpl" 
   unit-reg->param param->unit-reg keV :slider 0 20000 1 1 "Test Generator (KeV)"}
set table(TESTGENENABLE) {testgenenable int-widget->param param->int-widget "TestGenEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Test Generator Enable"}
set table(RESETFIFO) {resetfifo int-widget->param param->int-widget "FIFO.Reset"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Reset FIFO"}
set table(LASTCARD) {lastcard int-widget->param param->int-widget "LastCard"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Last Card"}

#    register selection for parameter save function

set registersavepatterns {
   Enable
   CFD.Thresh CFD.Delay CFD.Width  
   FTSample ValSample  PDSGate
     Energy.SSEnable Energy.ROEnable
     Timing.SSEnable  Timing.ROEnable
   CCR.Amp CCR.InvertPDS CCR.TestGen PileupReject
   LastCard
}




# inspection tables must be in same order as entries in the :choice-stacks
# this is achieved by using these tables when generating the choices
# index number returned maps directly to the signal number to be written to 
# the Signal Select register

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}
set insp_table(MUX-VI)  {"Insp.V"  VI}

set insp_menu(MUX-AI1) analogueinsptable
set insp_menu(MUX-AI2) analogueinsptable
set insp_menu(MUX-DI1) digitalinsptable
set insp_menu(MUX-DI2) digitalinsptable
set insp_menu(MUX-VI)  voltageinsptable


set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2 MUX-VI}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2 Insp.V}

#    the inspection line tables are in parallel
#    for channel select * implies value determined by adc channel
#    for parameter select * implies disconnect

set analogueinsptable {Disconnected
        Energy-Input TFA-Output LA-Output PDS-Output TAC-Output
}
set AI-ChanSelect {0 * * * * *}
set AI-Param      {* 0 1 2 3 4}


set digitalinsptable {Disconnected
        CFD-Direct CFD-Delay TAC-Start TAC+PDS-Reset Encode-Command PDS-Gate FT-Sample Val-Sample
        Fast_Trigger Inhibit Validation Veto OrValack OrDeadTime
        StartReadoutSIROP
        Renin(SIROP) SIROPDTready0 SIROPDTready1 SIROPDTready2 SIROPDTready3
        SIROPDTack_buf SIROPEmpty0 SIROPEmpty1 SIROPEmpty2 SIROPEmpty3
        SIROPLPASS
        CSA*(FifoChipSelect_A) CLKA(FifoClock_A) ENA(FifoEnablePort_A)
        AS*(VME) DS(VME) R/W*(VME) ModRoco
        CardRenin* BLTACK(Card) BLTACK(Bus) OE_Ident*
        CSB*(FifoChipSelect_B) CLKB(FifoClock_B) ENB(FifoEnablePort_B) OR_Fifo(EmptyFlag)
        FDTACK DTACK*(VME) CardRenout SysClk LT_Reset* Link_Ack Reset_Fifo* Decode_Sel
        En_DMA_Fifo Link_Clk En_DMA_Sirop* Evt_Cpt_Oe Link_Start Oe_Link_Out  
        DMA_Grant DMA_Req Link_Data0 Link_Data1 Link_Data2 Link_Data3
}

set DI-ChanSelect {0 * * * * * * * * 
        32 32 32 32 32 32 32 32
        32 32 32 32 32
        32 32 32 32 32
        32 32 32 32 32 32 32 32
        32 32 32 32 32 32 32 32
        32 32 32 32 32 32 32 32
        32 32 32 32 32
        32 32 32 32 32 32 
}

set DI-Param      {* 0 1 2 3 4 5 6 7
        1 2 3 4 5 6 7 8
        9 10 11 12 13
        14 15 16 17 18
        19 20 21 22 23 24 25 26
        27 28 29 30 31 32 33 34
        35 36 37 38 39 40 41 42
        43 44 45 46 47
        48 49 50 51 52 53
}

set voltageinsptable {Disconnected
        CFD-Delay CFD-Threshold CFD-Width Val-Sample FT-Sample PDS-Gate 
        +5V -5.2V +12V -12V +24V -24V -2V +20V 
        REFH
}
set VI-ChanSelect {0 * * * * * *  32 32 32 32 32 32 32 32 32 }
set VI-Param      {* 0 1 2 3 4 5  0  1  2  3  4  5  6  7  8}


set digitalinspmenu {Disconnected
        CFD-Direct CFD-Delay TAC-Start TAC+PDS-Reset Encode-Command PDS-Gate FT-Sample Val-Sample
        {"GIR lines #1" Fast_Trigger Inhibit Validation Veto OrValack OrDeadTime}
        {"SIROP lines"  StartReadoutSIROP
        Renin(SIROP) SIROPDTready0 SIROPDTready1 SIROPDTready2 SIROPDTready3
        SIROPDTack_buf SIROPEmpty0 SIROPEmpty1 SIROPEmpty2 SIROPEmpty3 SIROPLPASS}
        {"GIR lines #2" CSA*(FifoChipSelect_A) CLKA(FifoClock_A) ENA(FifoEnablePort_A)
        AS*(VME) DS(VME) R/W*(VME) ModRoco
        CardRenin* BLTACK(Card) BLTACK(Bus) OE_Ident*
        CSB*(FifoChipSelect_B) CLKB(FifoClock_B) ENB(FifoEnablePort_B) OR_Fifo(EmptyFlag)
        FDTACK DTACK*(VME) CardRenout SysClk LT_Reset* Link_Ack Reset_Fifo* Decode_Sel
        En_DMA_Fifo Link_Clk En_DMA_Sirop* Evt_Cpt_Oe Link_Start Oe_Link_Out  
        DMA_Grant DMA_Req Link_Data0 Link_Data1 Link_Data2 Link_Data3}
}


#    these are common
set steplist1 "CFDWIDTH"
#    these are per segment
set steplist2 "CFDDELAY CFDTHRESH  FTSAMPLE VALSAMPLE "


loadsource gir
loadsource common.vxi


proc click arg {
   global debug userselection allchans
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      *REDISPLAY   {click-redisplay}
      ENGINEERING  {click-$v}
      FORMAT      {click-format}
      STEP        {click-step $v $i}
      
      GIR*         {click-GIRwindow $w $v}

      CHANNEL     {click-channel $v}
      ALLCHANS    {click-whichchans $v}

      CHANSEL   {click-chanselection $v}

      PARAM       {click-param $v}

      MUX-*       {click-inspection $w [lindex $arg 4]}

      default   {
          if {$allchans < 0} then {set userselection 1}
             click-on $w $v
             set userselection 0
      }
   }
}

set electronicsframe "[self frame-name].electronics"

proc make-frame1 {} {
  global env chanlist
  global analogueinsptable digitalinspmenu voltageinsptable stepmenu stepmode

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :choice-stack CHANNEL :label "Current Channel:" :strings $chanlist :valuex 18c
      :hskip 100
      :checkbox ENABLE :label "Enable:" :valuex 10c
    :next-row T
      :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0
    :row-gap 10; :col-gap 15
    :next-row T
      :choice AMP    :label "Amp:" :strings [list "Int" "Ext"]
      :choice INVERT :label "Invert PDS:" :strings [list "Off" "On"] 
      :choice CTESTGEN :label "Test Generator:" :strings [list Off On]
      
    :next-row T
      :choice MARKREJECT :label "Pre-Pileup:" :strings [list "Mark" "Reject"] 

   :panel P2 :layout T :padx 10
     :layout H; :col-gap 25
      :hskip 75
      :message M1  :label "SS"        :bold T
      :message M2  :label "RO"        :bold T
      :message M3  :label "Address"   :bold T
   :row-gap 5; :next-row T
      :checkbox SSE      :label "Energy:" :valuex 12c
      :checkbox READOUTE :label " "
      :text ADDRE        :label " " :w 6   :read-only T 
   :next-row T
      :checkbox SST       :label "Timing:" :valuex 12c
      :checkbox READOUTT  :label " "
      :text ADDRT         :label " " :w 6   :read-only T 
      
   :panel P3 :layout T :padx 10 :pady 10
     :layout V; :row-gap 5
      :slider CFDWIDTH  :label "CFD Width (ns):" :min 0 :max 550   :valuex 30c
      :slider CFDDELAY  :label "CFD Delay (ns):" :min 0 :max 550   :valuex 30c
      :slider CFDTHRESH :label "CFD Threshold:" :min 0 :max 255 :valuex 30c
      :slider FTSAMPLE  :label "Fast Trigger Sample (ns):" :min 0 :max 3200   :valuex 30c
      :slider VALSAMPLE :label "Validation Sample (ns):" :min 0 :max 10000   :valuex 30c
      :slider PDSGATE   :label "PDS Gate (ns):" :min 0 :max 10000   :valuex 30c

      :vskip 10
      :layout H; :col-gap 10; :row-gap 10
      :next-row T
      :menubutton MUX-AI1 :label "Analogue Inspection 1:" :w 22 :menu $analogueinsptable
      :message MUX-AI1-USE   :label ""
      :next-row T
      :menubutton MUX-AI2 :label "Analogue Inspection 2:" :w 22 :menu $analogueinsptable
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital Inspection 1:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital Inspection 2:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI2-USE   :label ""
      :next-row T
      :menubutton MUX-VI  :label "Voltage Inspection:"    :w 22 :menu $voltageinsptable
      :message MUX-VI-USE   :label ""

   :panel P4 :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 20; :row-gap 5
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software]
      GIRbutton
      :next-row T
      :button REDISPLAY
      :menubutton STEP      :label "[lindex $stepmenu $stepmode]"  :strings $stepmenu 
      :button FORMAT :label "By Parameter"
      :hskip 20
      :button HELP  
   
 :show T

}

proc make-frame2 {} {
  global env parameternames

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

   :panel P10 :layout T :padx 5 :pady 5
     :layout H
      :choice-stack PARAM :label "Parameter:" :valuex 15c :strings $parameternames
      :hskip 200
      :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0
   
   :panel PANEL :layout T :padx 5

   :panel P11 :layout B  :padx 5 :after FOOTER
     :layout H
      :button REDISPLAY
      :button FORMAT :label "By Channel"
      :hskip 80
      :button HELP :command click-help
   
   :show T
}

proc make-electronics {} {
  global env electronicsframe electronicsvisible

  :frame "$electronicsframe" :label "Saphir Electronics" :resize F :transient T

   :panel P20 :layout T :padx 5 :pady 5
     :layout V; :row-gap 5
      :checkbox LASTCARD  :label "Last Card:"                            :valuex 22c
      :text MODELCODE     :label "Model Code:" :w 6  :read-only T        :valuex 22c
      :text MANID         :label "Manufacturer ID:" :w 6  :read-only T   :valuex 22c
      :text SERIALNUM     :label "Serial Number:" :w 6 :read-only T      :valuex 22c
      :checkbox RESETFIFO :label "Reset FIFO:"                           :valuex 22c
      :text MODLEVEL      :label "Modification Level:" :w 6 :read-only T :valuex 22c
      :checkbox TESTGENENABLE      :label "Test Gen Enable:"             :valuex 22c
      :text OFFSET        :label "Offset Register:" :w 6 :read-only T    :valuex 22c
      :slider TESTGEN     :label "Test Generator (KeV):" :min 0 :max 20000 :resolution 1 :valuex 22c
   
  :show T
 set electronicsvisible 1
}

proc make-chanSELframe {} {
  global chansel chanlist

   :frame CHANSEL-FRAME :label "Saphir Channel Selection"  :show-footer F :resize F :transient T
   :panel CHANSEL-PANEL :layout T
   :choice CHANSEL :exclusive F :strings $chanlist :value $chansel
   :show T
}  
